#!/usr/bin/perl
=begin comment info
 +-----------------------------------------------------------------------------+
 | Copyright 2001, Guardian Digital Inc.
 | $Id: config_lids.pl,v 1.3 2002/12/12 18:50:23 rwm Exp $
 |
 | Ryan W. Maple, <ryan@guardiandigital.com>
 +-----------------------------------------------------------------------------+
=end comment info
=cut

  # use perl;
  use Crypt::RIPEMD160;
  use Getopt::Long;
  use POSIX qw(setsid);
  use strict;

  # Set up some constants.
  use constant NUM_STEPS      => 7;
  use constant SBINDIR        => '/sbin/';
  use constant SYSCONFDIR     => '/etc/';

  use constant HTTPD_BIN      => '/usr/sbin/httpd';
  use constant HTTPD_BASEDIR  => '/home/httpd';
  use constant WEBTOOL_SCRIPT => '/usr/webtool/miniserv.pl';
  use constant WEBTOOL_CONF   => '/etc/webtool/SystemManager/config';

  use constant LIDS_RCDIR     => SYSCONFDIR . 'lids.d';
  use constant LIDSADM        => SBINDIR    . 'lidsadm';
  use constant LIDSADM_GD     => SBINDIR    . 'lidsadm_gd';
  use constant LIDSCONF       => SBINDIR    . 'lidsconf';

  # Set this to >= 1 for debugging information.
  use constant DEBUG => 0;

  # Define global variables.
  use vars qw($opt_f $opt_u @DENY @READ @WRITE @OBJECTS);

  # Subroutine prototypes.
  sub DBG($@);
  sub DIE($);
  sub Usage();
  sub execute($@);
  sub get_lidspass();
  sub is_lids_enabled();
  sub verify($);
  sub do_read_files();
  sub do_clear_rules($);
  sub do_protect_READ($);
  sub do_protect_WRITE($);
  sub do_protect_DENY($);
  sub do_protect_associations($);
  sub do_protect_ssl($);
  sub do_reload_config($);

  # Parse options.
  GetOptions ('force' => \$opt_f, 'help'  => \$opt_u);

  # Die if the user wants usage information.
  Usage() if ($opt_u);

  # Basic sanity checks.
  DIE(LIDSADM    . " does not exist or is not executable.") unless (-x LIDSADM);
  DIE(LIDSADM_GD . " does not exist or is not executable.") unless (-x LIDSADM_GD);
  DIE(LIDSCONF   . " does not exist or is not executable.") unless (-x LIDSCONF);

  if (! $opt_f && is_lids_enabled()) {
    DIE("This script cannot run because LIDS is enabled globally.");
  }


################################################################################
# # # # # # # # # # DO NOT CHANGE ANYTHING BELOW THIS LINE # # # # # # # # # # #
################################################################################

  #
  # These are all done as subroutines since sometimes a user may want to play
  # around with the ordering of the rules.  Rearrange (and modify argument) as
  # you see fit.
  #
  do_read_files();
  do_clear_rules(1);
  do_protect_READ(2);
  do_protect_WRITE(3);
  do_protect_DENY(4);
  do_protect_associations(5);
  do_protect_ssl(6);
  do_reload_config(7);

  # Exit quietly.
  print "\n";
  exit 0;


################################################################################
# Process config files.  These files reside in LIDS_RCDIR and are of the form
#
#   [0-9]+-[A-Za-z\_\-]+
#
sub do_read_files() {
  chdir LIDS_RCDIR or die "Could not chdir() ", LIDS_RCDIR, ": $!\n";
  opendir RCFILES, '.' or die "Could not read ", LIDS_RCDIR, ": $!\n";
  foreach my $FILE (sort readdir RCFILES) {
    next unless ($FILE =~ m/^[0-9]+\-[A-Za-z\_\-]+$/);

    do $FILE;

    push @DENY,    &getDENY();
    push @READ,    &getREAD();
    push @WRITE,   &getWRITE();
    push @OBJECTS, &getOBJECTS();
  }
  closedir RCFILES;
}

################################################################################
# Clear any existing rules.
#
sub do_clear_rules($) {
  my $step = shift;

  print "Configuring the LIDS subsystem...\n";
  print "  * [$step/", NUM_STEPS, "] Clearing existing rules.\n";
  execute(LIDSCONF, '-Z');
}

################################################################################
# Set up READ permissions.
#
sub do_protect_READ($) {
  my $step = shift;

  print "\n" if (DEBUG > 0);
  print "  * [$step/", NUM_STEPS, "] Setting up READ-ONLY permissions.\n";
  foreach my $file (sort @READ) {
    DBG(2, "SET: READONLY, $file");

    if (! verify($file)) {
      execute(LIDSCONF, '-A', '-o', $file , '-j', 'READONLY');
    }
  }
}

################################################################################
# Set up WRITE permissions.
#
sub do_protect_WRITE($) {
  my $step = shift;

  print "\n" if (DEBUG > 0);
  print "  * [$step/", NUM_STEPS, "] Setting up WRITE permissions.\n";
  foreach my $file (sort @WRITE) {
    DBG(2, "SET: WRITE, $file");

    if (! verify($file)) {
      execute(LIDSCONF, '-A', '-o', $file , '-j', 'WRITE');
    }
  }
}

################################################################################
# Set up DENY permissions.
#
sub do_protect_DENY($) {
  my $step = shift;

  print "\n" if (DEBUG > 0);
  print "  * [$step/", NUM_STEPS, "] Setting up DENY permissions.\n";
  foreach my $file (sort @DENY) {
    DBG(2, "SET: DENY, $file");

    if (! verify($file)) {
      execute(LIDSCONF, '-A', '-o', $file , '-j', 'DENY');
    }
  }
}

################################################################################
# Set up subject/object assocations.
#
sub do_protect_associations($) {
  my $step = shift;

  print "\n" if (DEBUG > 0);
  print "  * [$step/", NUM_STEPS, "] Setting up SUBJECT/OBJECT relationships.\n";
  foreach my $obj (sort @OBJECTS) {
    DBG(2, "SET: src = $obj->{'SOURCE'}, obj = $obj->{'OBJECT'}, prot = $obj->{'PROTECTION'}");

    if (! verify($obj->{'SOURCE'}) && ! verify($obj->{'OBJECT'})) {
      my @args;
      push @args, '-A';
      push @args, '-s';
      push @args, $obj->{'SOURCE'};
      push @args, '-o';
      push @args, $obj->{'OBJECT'};
      push @args, $obj->{'FLAG'} if ($obj->{'FLAG'});
      push @args, '-j';
      push @args, $obj->{'PROTECTION'};

      execute(LIDSCONF, @args);
    }
  }
}

################################################################################
# DENY all SSL certificates and keys to everybody except Apache.
#
sub do_protect_ssl($) {
  my $step = shift;

  print "\n" if (DEBUG > 0);
  print "  * [$step/", NUM_STEPS, "] Protecting SSL certificates and keys.\n";

  chdir HTTPD_BASEDIR or die "Could not chdir() ", HTTPD_BASEDIR, ": $!\n";
  opendir SITES, '.' or die "Could not read ", HTTPD_BASEDIR, ": $!\n";
  foreach my $dir (sort readdir SITES) {
    if (-f "$dir/ssl/certificate.pem" && -f "$dir/ssl/key.pem") {
      DBG(2, "Hide $dir/ssl");
      execute(LIDSCONF, '-A', '-o', "$dir/ssl",
                              '-j', 'DENY');

      DBG(2, "Let Apache READ $dir/ssl");
      execute(LIDSCONF, '-A', '-s', HTTPD_BIN,
                              '-o', "$dir/ssl",
                              '-j', 'READONLY');

      DBG(2, "Let the WebTool WRITE to $dir/ssl");
      execute(LIDSCONF, '-A', '-s', WEBTOOL_SCRIPT,
                              '-o', "$dir/ssl",
                              '-i', '20',
                              '-j', 'WRITE');
    }
  }
}

################################################################################
# Reload LIDS configuration.
#
sub do_reload_config($) {
  my $step = shift;

  print "\n" if (DEBUG > 0);
  print "  * [$step/", NUM_STEPS, "] Reloading LIDS configuration.\n";
  my $pass = get_lidspass();
  unless ($pass) {
    print STDERR "ERROR: You must reload your LIDS configuration manually\n";
    exit 1;
  }
  execute(LIDSADM_GD, '-Q', '--', '+RELOAD_CONF', $pass);
}



################################################################################
sub DBG($@) {
  my $level = shift;
  my $msg   = shift;
  my $disp  = shift || 'DBG';

  if (DEBUG >= $level)  { print "        - [$disp] $msg\n"; }
}

################################################################################
sub DIE($) {
  my $msg = shift;

  print "The following errors were encountered:\n";
  print "  - $msg\n";
  exit 2;
}

################################################################################
sub Usage() {
  print STDERR "Usage: $0 { --force | --help }\n";
  exit 0;
}

################################################################################
sub execute($@) {
  my $rv;
  my $cmd  = shift;
  my @args = @_;
  my $pid  = fork();

  if ($pid == 0) {
    setsid();

    open STDIN,  '/dev/null' or die "Could not open /dev/null: $!\n";
    open STDOUT, '/dev/null' or die "Could not open /dev/null: $!\n";
    open STDERR, '/dev/null' or die "Could not open /dev/null: $!\n";

    exec $cmd, @args;
    exit;
  }
  elsif (! $pid) {
    print "Could not fork(): $!\n";
    exit -1;
  }
  else {
    wait();
    $rv = $?;
  }

  if ($rv != 0) {
    my $line = $cmd; foreach my $a (@args) { $line .= " $a"; }
    DBG(0, "exec(): RV = $rv, CMD = $line", 'ERROR');
  }

  return $rv;
}

################################################################################
sub is_lids_enabled() {
  my $rv = 0;

  open LIDS, LIDSADM . ' -V |' or return 1;
  while (my $line = <LIDS>) {
    if ($line =~ m/LIDS_GLOBAL\s+([0-9])$/) { $rv = $1; }
  }
  close LIDS;

  return $rv;
}

################################################################################
sub get_lidspass() {
  my $rv = undef;

  if ( -f WEBTOOL_CONF) {
    open FILE, WEBTOOL_CONF;
    while (<FILE>) { if ($_ =~ m/^pass\=(.+)$/) { $rv = $1; } }
    close FILE;
  }

  return undef unless ($rv);

  my $tmp = new Crypt::RIPEMD160;

  $tmp->add($rv);
  $rv = $tmp->digest();
  $rv = unpack "H*", $rv;

  return $rv;
}

################################################################################
sub verify($) {
  my $file = shift;

  if ($file =~ m/^CAP_/) { return 0; }
  elsif (! -e $file) {
    DBG(1, "$file: No such file or directory", 'ERROR');
    return 1;
  }

  return 0;
}
